import re
from whatchamacallit import make


def test_read_file():
    spec_map = ('gridchen/', '/root/')
    src = """
    import 'gridchen/foo.js'
    import 'gridchen/bar.js'
    """
    t = re.sub(r'(import.*?) (["\'])' + spec_map[0], r'\1 \2' + spec_map[1], src, flags=re.DOTALL)

    expected = """
    import '/root/foo.js'
    import '/root/bar.js'
    """
    assert t == expected


def test_comment():
    src = """
    /*
    Comment 1
    */
    // Comment 2
    /*
    Comment 3
    */
    import "foo";
    import "bar";
    function foo() {
    }
    """
    m = re.match(r'(\s*(/\*.*?\*/|//[^\r\n]*))*', src, flags=re.DOTALL)
    print(m.group())
    # index = 0
    # for m in re.finditer(r'(\s*(/\*.*?\*/|//[^\r\n]*))*', src, flags=re.DOTALL):
    #     print(m)
    #     index = m.end()
    #     print(m.group())
    end_of_comments = m.end()
    m = re.search(r'(function|const|let|var)\s', src[end_of_comments:], flags=re.DOTALL)
    print(m)
    imports = src[end_of_comments:end_of_comments + m.start()]
    is_import_section = re.match('\s*import\s', imports) is not None
    if is_import_section:
        pass


def test_process_js_imports():
    imports = {
        "gridchen/": "https://decatur.github.io/grid-chen/gridchen/"
    }

    src = """
/*
Comments...
*/

import 'gridchen/foo.js'
import * from 'gridchen/foo.js'
export * from 'gridchen/foo.js'

function foo() {}
    """
    expected = """
/*
Comments...
*/

import 'https://decatur.github.io/grid-chen/gridchen/foo.js'
import * from 'https://decatur.github.io/grid-chen/gridchen/foo.js'
export * from 'https://decatur.github.io/grid-chen/gridchen/foo.js'

function foo() {}
    """
    mapped, was_mapped = make.process_imports(src, imports)
    print(mapped)
    assert mapped == expected


def test_process_html_imports():
    imports = {
        "gridchen/": "https://decatur.github.io/grid-chen/gridchen/"
    }

    src = """<!DOCTYPE html>
<html lang="en">

<head>
    <meta charset="UTF-8">
    <title>form-chen Master-Detail</title>
    <link rel="stylesheet" href="gridchen/demo.css">
    <script src="gridchen/theme.js"></script>
</head>

<body>
    <h1><a href="index.html">form-chen</a> / Master-Detail Demo</h1>
    <img src="gridchen/foo.png">
</body>
<script nomodule>alert('Your browser is not supported')</script>
<script type="module">
    import * as demo from "gridchen/demo.js"

    const schema = {};
</script>

</html>"""

    expected = """<!DOCTYPE html>
<html lang="en">

<head>
    <meta charset="UTF-8">
    <title>form-chen Master-Detail</title>
    <link rel="stylesheet" href="https://decatur.github.io/grid-chen/gridchen/demo.css">
    <script src="https://decatur.github.io/grid-chen/gridchen/theme.js"></script>
</head>

<body>
    <h1><a href="index.html">form-chen</a> / Master-Detail Demo</h1>
    <img src="https://decatur.github.io/grid-chen/gridchen/foo.png">
</body>
<script nomodule="None">alert('Your browser is not supported')</script>
<script type="module">
    import * as demo from "https://decatur.github.io/grid-chen/gridchen/demo.js"

    const schema = {};
</script>

</html>"""
    mapped, was_mapped = make.process_imports_html(src, imports)
    # print(mapped)
    assert mapped == expected




import pathlib
import importlib.util

from starlette.exceptions import HTTPException
from starlette.responses import HTMLResponse, Response

from whatchamacallit import make
from fastapi import FastAPI
from fastapi.staticfiles import StaticFiles


class JavascriptResponse(Response):
    media_type = "text/javascript"


app = FastAPI()


@app.get(r"/{file_path:path}\.js", response_class=JavascriptResponse)
def read_js(file_path: str):
    source = pathlib.Path(file_path + '.js')
    if len(source.parts) > 1:
        prefix = source.parts[0].replace('@', '')
        spec = importlib.util.find_spec(prefix)
        if spec:
            origin = pathlib.Path(spec.origin)
            source = origin.parent.parent / source

    return make.read_file(source, {'gridchen/': '/gridchen/'})


@app.get(r"/{file_path:path}\.html", response_class=HTMLResponse)
def read_root(file_path: str):
    source = pathlib.Path(file_path + '.html')
    if not source.is_file():
        raise HTTPException(status_code=404, detail="Item not found")
    return make.read_file(source, {'gridchen/': '/gridchen/'})


app.mount("/", StaticFiles(directory="."), name="static")



import importlib
import re
import shutil
from html.parser import HTMLParser
from pathlib import Path
from typing import List, Tuple, Generator, Dict, Optional


# Bundlers:
# Unpkg.com, rollup, webpack, babel, pika, assetgraph, Browserify,
# gulp, JSPM
# In Place:
# snowflake, Open Web Components, browser sync, es-module-shims
# See
#   http://dplatz.de/blog/2019/es6-bare-imports.html
#   https://jakearchibald.com/2017/es-modules-in-browsers/
#   https://medium.com/@dmnsgn/in-2020-go-bundler-free-eb29c1f05fc9
#   https://wicg.github.io/import-maps/
#   https://medium.com/@dmnsgn/es-modules-in-the-browser-almost-now-3638ffafdc68


def process_file(source: Path, target: Path, spec_mapping: Dict[str, str]):
    if not target.parent.is_dir():
        target.parent.mkdir(parents=True)
    if source.suffix in {'.html', '.js'}:
        t = read_file(source, spec_mapping)
        target.write_text(t, encoding='utf8')
    else:
        shutil.copy2(source.as_posix(), target.as_posix())


def read_file(source: Path, imports: Dict[str, str]) -> str:
    print('read_file ' + source.as_posix())
    src = source.read_text(encoding='utf8')
    processor = process_imports if source.suffix == '.js' else process_imports_html
    src, was_patched = processor(src, imports)
    # TODO: Cache processed modules
    return src


def process_imports(src: str, imports: Dict[str, str], scopes: dict = None) -> Tuple[str, bool]:
    """
    Remaps the imports and exports in the specified Javascript module according the import map.
    This is the Python equivalence to https://github.com/guybedford/es-module-shims for Javascript.

    :param src:
    :param imports: See https://github.com/WICG/import-maps
    :param scopes: Not used. See https://github.com/WICG/import-maps
    :return:
    """

    # Skip initial comments. Note this always matches.
    m = re.match(r'(\s*(/\*.*?\*/|//[^\r\n]*))*', src, flags=re.DOTALL)
    # print(m.group())
    # index = 0
    # for m in re.finditer(r'(\s*(/\*.*?\*/|//[^\r\n]*))*', src, flags=re.DOTALL):
    #     print(m)
    #     index = m.end()
    #     print(m.group())
    end_of_comments = m.end()
    m = re.search(r'(function|const|let|var)\s', src[end_of_comments:], flags=re.DOTALL)
    if m is None:
        # TODO: Handle empty module
        return src, False

    import_section = src[end_of_comments:end_of_comments + m.start()]
    is_import_section = re.match(r'\s*(import|export)\s', import_section) is not None
    if is_import_section:
        was_patched = False
        for key, value in imports.items():
            import_section, patch_count = re.subn(
                r'((import|export).*?) (["\'])' + key,
                r'\1 \3' + value,
                import_section, flags=re.DOTALL)
            was_patched |= patch_count > 0

        if was_patched:
            return src[0:end_of_comments] + import_section + src[end_of_comments + m.start():], was_patched
        else:
            return src, False
    else:
        return src, False


class MyHTMLParser(HTMLParser):
    def error(self, message):
        raise Exception(message)

    def __init__(self, import_mappings):
        super().__init__()
        self.import_mappings = import_mappings
        self.is_script = False
        self.buffer: List[str] = []
        self.was_patched = False

    def print(self, data):
        self.buffer.append(data)

    def process_ref(self, ref: str):
        parts = ref.split('/')
        if len(parts) > 1 and parts[0] + '/' in self.import_mappings:
            return ref.replace(parts[0] + '/', self.import_mappings[parts[0] + '/'])
        return ref

    def handle_starttag(self, tag: str, attrs):
        tag = tag.lower()
        self.is_script = (tag == "script")
        self.print('<' + tag)
        for key, value in attrs:
            if key in {'src', 'href', 'cite', 'data', 'srcset'}:
                value = self.process_ref(value)
            self.print(f' {key}="{value}"')
        self.print('>')

    def handle_endtag(self, tag):
        self.is_script = False
        self.print('</' + tag + '>')

    def handle_data(self, data):
        if self.is_script:
            data, was_patched = process_imports(data, self.import_mappings)
            self.was_patched |= was_patched

        self.print(data)

    def handle_decl(self, data):
        self.print(f'<!{data}>')


def process_imports_html(src:str, imports: Dict[str, str], scopes: dict = None) -> Tuple[str, bool]:
    parser = MyHTMLParser(imports)
    parser.feed(src)
    return ''.join(parser.buffer), False


def process_dir(source: Generator[Path, None, None], target_root: Path, spec_mapping: Dict[str, str]):
    for elem in source:
        if elem.is_file():
            process_file(elem, target_root / elem, spec_mapping)


def process_dir_recursive(source: Path, target_root: Path, spec_mapping: Dict[str, str]):
    for elem in source.rglob('*.*'):
        p = Path('./') / elem
        if p.is_file():
            process_file(p, target_root / elem, spec_mapping)


def resolve(path) -> Optional[Path]:
    if len(path.parts) == 1 or not path.suffix:
        return
    prefix = path.parts[0].replace('@', '')
    # Only finds the spec if package contains a __init__.py!
    spec = importlib.util.find_spec(prefix)
    if not spec:
        return

    # spec.origin = WindowsPath('C:/ws/projects/VirtualPowerStorage/venv/lib/site-packages/appchen/__init__.py')
    origin = Path(spec.origin)
    source = origin.parent.parent / prefix / Path(*path.parts[1:])
    return source



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['whatchamacallit']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'whatchamacallit',
    'version': '0.0.2',
    'description': 'Helper for In Place web servers.',
    'long_description': '# whatchamacallit\n\nUse web resources the pythonic way. Specify Javascript modules, HTML, CSS as bare URLs and rewrite on the fly.\n\n# Examples\n\n\n# Configure Poetry\n\n````shell script\npoetry publish\n````\n\n\n# Build\n\n````shell script\npoetry build\n````\n\n# Publishing to PyPI\n\n````shell script\npoetry publish\n````\n\n# Previous Work\n\n\n# Bundlers:\n# Unpkg.com, rollup, webpack, babel, pika, assetgraph, Browserify, gulp, JSPM\n# In Place:\n# snowflake, Open Web Components, browser sync, es-module-shims\n# See\n#   http://dplatz.de/blog/2019/es6-bare-imports.html\n#   https://jakearchibald.com/2017/es-modules-in-browsers/\n#   https://medium.com/@dmnsgn/in-2020-go-bundler-free-eb29c1f05fc9\n#   https://wicg.github.io/import-maps/\n#   https://medium.com/@dmnsgn/es-modules-in-the-browser-almost-now-3638ffafdc68\n',
    'author': 'Wolfgang Kühn',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/decatur/whatchamacallit',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import logging
import uvicorn

# Same as command line (see https://www.uvicorn.org/#command-line-options)
# uvicorn dashboard.main:app --host 127.0.0.1 --port 8089 --log-level debug
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

if __name__ == '__main__':
    uvicorn.run("whatchamacallit.fastapi_server:app", host="0.0.0.0", port=8000, access_log=True, log_level="trace", workers=1, debug=False)